## TurnGen: a household chores turn generator
## Copyright (C) 2006 Angelo "Encelo" Theodorou
##

import os
import datetime
from pysqlite2 import dbapi2 as sqlite

import gettext
gettext.bindtextdomain('turngen', 'po')
gettext.textdomain('turngen')
_ = gettext.gettext


class Database:
	"""Internal database managing class"""
	def __init__(self, fname, config):
		"""Connect and try to create basic tables"""
		if os.path.exists(fname) == False:
			print _('Creating ') + fname
			self.con = sqlite.connect(fname)
			self.cur = self.con.cursor()
			self.create_tables()
			self.init_tables(config)
			self.con.commit()
		else:
			self.con = sqlite.connect(fname)
			self.cur = self.con.cursor()


	def __del__(self):
		"""Close the connection"""
		self.con.close()


	def create_tables(self):
		"""Create users, activities and data table"""

		CREATE_users = "CREATE TABLE users (id INTEGER NOT NULL, name VARCHAR(128));"
		CREATE_activities = "CREATE TABLE activities (id INTEGER NOT NULL, name VARCHAR(128));"
		CREATE_data = "CREATE TABLE data (id INTEGER NOT NULL, presence VARCHAR(128) NOT NULL, user INTEGER NOT NULL, activity INTEGER NOT NULL, date TIMESTAMP NOT NULL);"
		self.cur.execute(CREATE_users)
		self.cur.execute(CREATE_activities)
		self.cur.execute(CREATE_data)
		print _('Tables created')


	def init_tables(self, config):
		"""Init tables"""
		users = config['Users']
		activities = config['Activities']

		INSERT_users = "INSERT INTO users VALUES (%s, '%s');"
		INSERT_activities = "INSERT INTO activities VALUES (%s, '%s');"

		id = 0
		for user in users:
			self.cur.execute(INSERT_users % (id, user))
			id += 1

		id = 0
		for activity in activities:
			self.cur.execute(INSERT_activities % (id, activity))
			id += 1

		print _('Tables initialized')


	def get_user_id(self, name):
		"""Return id of the user"""

		SELECT_user = "SELECT id FROM users WHERE name='%s';" % name

		self.cur.execute(SELECT_user)
		fetch = self.cur.fetchall()

		if fetch == []:
			return None
		else:
			return fetch[0][0]


	def get_activity_id(self, name):
		"""Return id of the activity"""

		SELECT_activity = "SELECT id FROM activities WHERE name='%s';" % name

		self.cur.execute(SELECT_activity)
		fetch = self.cur.fetchall()

		if fetch == []:
			return None
		else:
			return fetch[0][0]


	def get_user_name(self, id):
		"""Return name of the user"""

		SELECT_user = "SELECT name FROM users WHERE id=%s;" % id

		self.cur.execute(SELECT_user)
		fetch = self.cur.fetchall()

		if fetch == []:
			return None
		else:
			return fetch[0][0]


	def get_activity_name(self, id):
		"""Return name of the activity"""

		SELECT_activity = "SELECT name FROM activities WHERE id=%s;" % id

		self.cur.execute(SELECT_activity)
		fetch = self.cur.fetchall()

		if fetch == []:
			return None
		else:
			return fetch[0][0]


	def get_laziest_users_id(self, presence_l, activity_id):
		"""Return the id of the users who are the last in the activity list with the specified presence."""
		presence_l.sort()
		presence = self.enc_presence(presence_l)
		num_present = len(presence_l)
		COUNT_active_users = "SELECT COUNT(DISTINCT user) FROM data WHERE presence='%s' AND activity=%s;" % (presence, activity_id)
		SELECT_active_users = "SELECT DISTINCT user FROM data WHERE presence='%s' AND activity=%s;" % (presence, activity_id)
		SELECT_less_active_user = "SELECT user, COUNT(user) FROM data WHERE presence='%s' AND activity=%s GROUP BY user ORDER BY COUNT(user) ASC;" % (presence, activity_id)

		self.cur.execute(COUNT_active_users)
		fetch = self.cur.fetchall()

		if fetch == []:
			print _('No active users for "%s" where presence is "%s"') % (self.get_activity_name(activity_id), presence)
			return None
		else:
			num_active = fetch[0][0]

		if num_present > num_active: # There are users who have never done this activity, they are the laziest!
			temp = presence_l[:]
			self.cur.execute(SELECT_active_users)
			fetch = self.cur.fetchall()
			for row in fetch:
				temp.remove(row[0])
				
			return temp # Returning the first user, both if there is one or many
	
		elif num_present == num_active:
			self.cur.execute(SELECT_less_active_user)
			fetch = self.cur.fetchall()
			if fetch == []:
				print _('Something went wrong when selecting the less active user')
			else:
				# Should always return a list
				temp = []
				temp.append(fetch[0][0])
				return temp
		
		else:
			print _('Error in the database for "%s" where presence is "%s"') % (self.get_activity_name(activity_id), presence)
			print _('More active users than present one!')
			exit


	def get_laziest_user_id(self, presence_l, activity_id):
		"""Return only the first laziest user found"""

		users = self.get_laziest_users_id(presence_l, activity_id)

		return users[0]


	def __get_last_data_id(self):
		"""Return the last (greatest) id in the data table"""

		SELECT_last_id = "SELECT MAX(id) FROM data"

		self.cur.execute(SELECT_last_id)
		fetch = self.cur.fetchall()

		if fetch == []:
			return None
		else:
			return fetch[0][0]


	def print_fetch(self, fetch):
		"""Print a fetch from a database query"""

		for row in fetch:
			line = ''
			for i in range(len(row)-1):
				line = line + str(row[i]) + '|'
			line = line + str(row[len(row)-1])	
			print line


	def insert_data(self, presence_l, user_id, activity_id, separate=True):
		"""Insert an action into the data table"""
		now = str(datetime.datetime.now())
		#_now = datetime.datetime.now()
		#now = str(_now.year) + '-' + str(_now.month) + '-' + str(_now.day) + ' ' + str(_now.hour) + ':' + str(_now.minute)
		presence_l.sort()
		presence = self.enc_presence(presence_l)

		last_id = self.__get_last_data_id()
		INSERT_data = "INSERT INTO data VALUES (%s, '%s', %s, %s, '%s')"

		if last_id == None:
			id = 0
		else:
			id = last_id

		if separate == True:
			id += 1
		
		self.cur.execute(INSERT_data % (id, presence, user_id, activity_id, now))
		self.con.commit()

		return id


	def remove_data(self, data_id):
		"""Remove an action from the data table"""

		DELETE_id = "DELETE FROM data where id='%s';" % data_id

		self.cur.execute(DELETE_id)
		self.con.commit()


	def enc_presence(self, list):
		"""Encode a presence list in a string"""
		list.sort()
		string = ''

		for x in list:
			if list.index(x) == len(list)-1: # last element
				string += str(x)
			else:
				string += str(x) + '-'

		return string


	def dec_presence(self, string):
		"""Decode a presence string in a list"""

		list = string.split('-')
		list = [int(x) for x in list]
			
		return list


	def valid_user_id(self, id):
		"""Check if user id is valid"""

		SELECT_user = "SELECT id FROM users where id='%s'" % id

		self.cur.execute(SELECT_user)
		fetch = self.cur.fetchall()

		if fetch == []:
			return False
		else:
			return True


	def valid_activity_id(self, id):
		"""Check if activity id is valid"""
												
		SELECT_activity = "SELECT id FROM activities WHERE id='%s'" % id

		self.cur.execute(SELECT_activity)
		fetch = self.cur.fetchall()

		if fetch == []:
			 return False
	 	else:
			return True


	def valid_presence(self, presence_l):
		"""Check if the presence list is valid"""

		for x in presence_l:
			if self.valid_user_id(x) == False:
				return False

		return True


	def valid_data_id(self, id):
		"""Check if data id is valid"""

		SELECT_data = "SELECT id FROM data WHERE id='%s'" % id

		self.cur.execute(SELECT_data)
		fetch = self.cur.fetchall()

		if fetch == []:
			return False
		else:
			return True


	def fetch_table(self, name):
		"""Fetch a table"""

		SELECT_all = "SELECT * FROM %s" % name

		try:
			self.cur.execute(SELECT_all)
		except sqlite.Error:
			print _('No table named ') + name
		fetch = self.cur.fetchall()

		return fetch


	def fetch_joined_data(self):
		"""Fetch a join selected data table"""

		SELECT_join = """SELECT data.id, data.presence, users.name, activities.name, date FROM data INNER JOIN users, activities ON data.user=users.id AND data.activity=activities.id;"""

		self.cur.execute(SELECT_join)
		fetch = self.cur.fetchall()

		return fetch


	def fetch_ranking(self, presence_l, activity_id):
		"""Fetch an ordered table with users based on presence and activities"""
		presence_l.sort()
		presence = self.enc_presence(presence_l)

		#SELECT_order = """SELECT users.name, activities.name, COUNT(user) FROM data INNER JOIN users, activities ON users.id=user AND activities.id=activity WHERE presence='%s' AND activity=%s GROUP BY user ORDER BY COUNT(user) DESC;""" % (presence. activity_id)
		SELECT_order = """SELECT users.name, COUNT(user) FROM data INNER JOIN users ON users.id=user WHERE presence='%s' AND activity=%s GROUP BY user ORDER BY COUNT(user) DESC;""" % (presence, activity_id)

		self.cur.execute(SELECT_order)
		fetch = self.cur.fetchall()

		return fetch


	def fetch_row(self, name, id):
		"""Fetch a row from a table based on a id"""

		SELECT_row = """SELECT * FROM %s WHERE id=%s""" % (name, id)

		self.cur.execute(SELECT_row)
		fetch = self.cur.fetchall()

		return fetch
