## TurnGen: a household chores turn generator
## Copyright (C) 2006 Angelo "Encelo" Theodorou
##

import pygtk
pygtk.require('2.0')
import gtk.glade

import about_dlg, table_win, selection_win, ranking_win

class Main_Window:
	def __init__(self, glade_file, db):
		self.glade_file = glade_file
		self.db = db
		
		self.xml = gtk.glade.XML(glade_file, root="main_win")
		signals = {
			"on_main_win_destroy": self.on_main_win_destroy,
			"on_data_list_button_press_event": self.on_data_list_button_press_event,
			# Menu items
			"on_quit_activate": self.on_quit_activate,
			"on_users_activate": self.on_users_activate,
			"on_activities_activate": self.on_activities_activate,
			"on_add_action_activate": self.on_add_action_activate,
			"on_remove_action_activate": self.on_remove_action_activate,
			"on_laziest_activate": self.on_laziest_activate,
			"on_ranking_activate": self.on_ranking_activate,
			"on_update_activate": self.on_update_activate,
			"on_about_activate": self.on_about_activate,
		}
		self.xml.signal_autoconnect(signals)


		self.users_win = None
		self.activities_win = None

		# Getting widgets
		self.main_win = self.xml.get_widget("main_win")
		self.data_list = self.xml.get_widget('data_list')
		self.statusbar = self.xml.get_widget("statusbar")
		self.users_item = self.xml.get_widget("users")
		self.activities_item = self.xml.get_widget("activities")

		# Setting initial attributes
		self.main_win.resize(300, 300)

		# Init methods
		self.setup_data_list()
		self.update_data_list()


	def setup_data_list(self):
		"""Setup the main listview with data from the db"""
		self.cell = gtk.CellRendererText()

		# Columns
		self.column = gtk.TreeViewColumn(_('Id'))
		self.column.pack_start(self.cell, True)
		self.column.add_attribute(self.cell, 'text', 0)
		self.column.set_resizable(True)
		self.column.set_sort_column_id(0)
		self.data_list.append_column(self.column)

		self.column = gtk.TreeViewColumn(_('Presence'))
		self.column.pack_start(self.cell, True)
		self.column.add_attribute(self.cell, 'text', 1)
		self.column.set_resizable(True)
		self.column.set_sort_column_id(1)
		self.data_list.append_column(self.column)

		self.column = gtk.TreeViewColumn(_('User'))
		self.column.pack_start(self.cell, True)
		self.column.add_attribute(self.cell, 'text', 2)
		self.column.set_resizable(True)
		self.column.set_sort_column_id(2)
		self.data_list.append_column(self.column)

		self.column = gtk.TreeViewColumn(_('Activity'))
		self.column.pack_start(self.cell, True)
		self.column.add_attribute(self.cell, 'text', 3)
		self.column.set_resizable(True)
		self.column.set_sort_column_id(3)
		self.data_list.append_column(self.column)

		self.column = gtk.TreeViewColumn(_('Date'))
		self.column.pack_start(self.cell, True)
		self.column.add_attribute(self.cell, 'text', 4)
		self.column.set_resizable(True)
		self.column.set_sort_column_id(4)
		self.data_list.append_column(self.column)

		# Model
		self.liststore = gtk.ListStore(int, str, str, str, str)
		self.data_list.set_model(self.liststore)


	def update_data_list(self):
		"""Populate the data list with actions"""

		self.liststore.clear()

		self.data = self.db.fetch_joined_data()
		for row in self.data:
			self.liststore.append([row[0], row[1], row[2], row[3], row[4]])


	# Callbacks
	def on_main_win_destroy(self, widget, data=None):
		return gtk.main_quit()


	def on_users_win_destroy(self, widget, data=None):
		"""Fix toggle menu item when users window gets destroyed"""

		self.users_item.set_active(False)


	def on_activities_win_destroy(self, widget, data=None):
		"""Fix toggle menu item when activities window gets destroyed"""

		self.activities_item.set_active(False)


	def on_data_list_button_press_event(self, widget, event, data=None):
		if self.data_list.get_path_at_pos( int(event.x), int(event.y) ) != None:
			(path, column, x, y) = self.data_list.get_path_at_pos( int(event.x), int(event.y) )
			self.data_list.row_activated(path, self.data_list.get_column(0))


	# Menu items
	def on_quit_activate(self, event):
		return gtk.main_quit()


	def on_about_activate(self, event):
		"""Open the about dialog"""
		about_dlg.About_Dialog(self.glade_file)
		return


	def on_users_activate(self, event):
		"""Show or hide the users table window"""
	
		if self.users_item.get_active() == True and self.users_win == None:
			self.users_win = table_win.Table_Window(self.glade_file, self.db, 'users')
			self.users_win.table_win.connect('destroy', self.on_users_win_destroy)
		else:
			self.users_win.table_win.destroy()
			self.users_win = None
		return


	def on_activities_activate(self, event):
		"""Show or hide the activities table window"""
		
		if self.activities_item.get_active() == True and self.activities_win == None:
			self.activities_win = table_win.Table_Window(self.glade_file, self.db, 'activities')
			self.activities_win.table_win.connect('destroy', self.on_activities_win_destroy)
		else:
			self.activities_win.table_win.destroy()
			self.activities_win = None
		return


	def on_add_action_activate(self, event):
		"""Add an action to the database"""

		self.selection_win = selection_win.Selection_Window(self.glade_file, self.db)
		self.sig_id = self.selection_win.ok_button.connect("clicked", self.on_add_action_ok_clicked)


	def on_add_action_ok_clicked(self, event):
		
		self.selection_win.ok_button.disconnect(self.sig_id)
		self.selection_win.selection_win.destroy()

		presence_l = self.selection_win.selection['presence_l']
		user_id = self.selection_win.selection['user']
		activity_id = self.selection_win.selection['activity']

		success = False
		if self.db.valid_presence(presence_l):
			if self.db.valid_user_id(user_id):
				if self.db.valid_activity_id(activity_id):
					if user_id in presence_l: # A user which is not present can't do an action!
						id = self.db.insert_data(presence_l, user_id, activity_id)
						self.statusbar.push(1, _('Added action %s') % id)
						success = True

		if success == False:
			dlg = gtk.MessageDialog(type=gtk.MESSAGE_ERROR, buttons=gtk.BUTTONS_CLOSE, 
				message_format=_('Error, action not inserted!'))
			dlg.set_markup(_('<b>Error</b>, action not inserted!'))
			dlg.run()
			dlg.destroy()
		else:
			row = self.db.fetch_row('data', id)
			self.liststore.append([row[0][0], row[0][1], self.db.get_user_name(row[0][2]), self.db.get_activity_name(row[0][3]), row[0][4]])	


	def on_remove_action_activate(self, event):
		"""Remove an action from the database"""

		list_selection = self.data_list.get_selection()
		(model, iter) = list_selection.get_selected()
		if iter == None:
			return
		id = model.get_value(iter, 0)

		dlg = gtk.MessageDialog(type=gtk.MESSAGE_QUESTION, buttons=gtk.BUTTONS_YES_NO,
			message_format=_('Do you really want to delete action %s?') % id)
		dlg.set_markup(_('Do you really want to <b>delete</b> action %s?') % id)
		response = dlg.run()
		dlg.destroy()

		if response == gtk.RESPONSE_NO:
			return

		if self.db.valid_data_id(id) == True:
			self.db.remove_data(id)
			model.remove(iter)
			self.statusbar.push(1, _('Removed action %s') % id)
		else:
			dlg = gtk.MessageDialog(type=gtk.MESSAGE_ERROR, buttons=gtk.BUTTONS_CLOSE,
				message_format=_('Error, action not removed!'))
			dlg.set_markup(_('<b>Error</b>, action not removed!'))
			dlg.run()
			dlg.destroy()


	def on_laziest_activate(self, event):
		"""Query for the laziest"""

		self.selection_win = selection_win.Selection_Window(self.glade_file, self.db, users_combo=False)
		self.sig_id = self.selection_win.ok_button.connect("clicked", self.on_laziest_ok_clicked)


	def on_laziest_ok_clicked(self, event):

		self.selection_win.ok_button.disconnect(self.sig_id)
		self.selection_win.selection_win.destroy()

		presence_l = self.selection_win.selection['presence_l']
		activity_id = self.selection_win.selection['activity']

		users_id = self.db.get_laziest_users_id(presence_l, activity_id)
		users_string = ''
		for user_id in users_id:
			user = self.db.get_user_name(user_id)
			users_string += user
			if users_id.index(user_id) != len(users_id) - 1:
				users_string += ', '


		dlg = gtk.MessageDialog(type=gtk.MESSAGE_INFO, buttons=gtk.BUTTONS_CLOSE,
			message_format=_('The laziest users are: %s') % users_string)
		dlg.set_markup(_('The laziest users are: <b>%s</b>') % users_string)
		dlg.run()
		dlg.destroy()


	def on_ranking_activate(self, event):
		"""Show the ranking window"""

		self.selection_win = selection_win.Selection_Window(self.glade_file, self.db, users_combo=False)
		self.sig_id = self.selection_win.ok_button.connect("clicked", self.on_ranking_ok_clicked)


	def on_ranking_ok_clicked(self, event):

		self.selection_win.ok_button.disconnect(self.sig_id)
		self.selection_win.selection_win.destroy()

		presence_l = self.selection_win.selection['presence_l']
		activity_id = self.selection_win.selection['activity']

		self.ranking_win = ranking_win.Ranking_Window(self.glade_file, self.db, presence_l, activity_id)


	def on_update_activate(self, event):
		"""Update data list"""

		self.update_data_list()
		self.statusbar.push(1, _('Actions list updated'))
