## TurnGen: a household chores turn generator
## Copyright (C) 2006 Angelo "Encelo" Theodorou
##

import pygtk
pygtk.require('2.0')
import gtk.glade

class Selection_Window:
	"""Create a window for selection"""
	def __init__(self, glade_file, db, users_list=True, users_combo=True, activities_combo=True):
		self.glade_file = glade_file
		self.db = db
		
		self.xml = gtk.glade.XML(glade_file, root="selection_win")
		signals = { 
			"on_ok_button_clicked": self.on_ok_button_clicked,
			"on_selection_win_destroy": self.on_selection_win_destroy 
		}
		self.xml.signal_autoconnect(signals)

		self.selection = {'presence': None, 'user': None, 'activity': None}

		# Getting widgets
		self.selection_win = self.xml.get_widget("selection_win")
		self.presence_frame = self.xml.get_widget("presence_frame")
		self.user_frame = self.xml.get_widget("user_frame")
		self.activity_frame = self.xml.get_widget("activity_frame")
		self.users_list = self.xml.get_widget("users_list")
		self.users_combo = self.xml.get_widget("users_combo")
		self.activities_combo = self.xml.get_widget("activities_combo")
		self.ok_button = self.xml.get_widget("ok_button")

		# Setting initial attributes
		self.selection_win.resize(200, 300)
		 
		# Init Methods
		self.setup_users_list()
		if users_list:
			self.populate_users_list()
		else:
			self.presence_frame.set_sensitive(False)
			
		if users_combo:
			self.setup_users_combo()
			self.populate_users_combo()
		else:
			self.user_frame.set_sensitive(False)
			
		if activities_combo:
			self.setup_activities_combo()
			self.populate_activities_combo()
		else:
			self.activity_frame.set_sensitive(False)
		
		return


	def setup_users_list(self):
		"""Setup a users list"""
		cell = gtk.CellRendererText()

		# Columns
		column = gtk.TreeViewColumn('Id')
		column.pack_start(cell, True)
		column.add_attribute(cell, 'text', 0)
		column.set_resizable(True)
		column.set_sort_column_id(0)
		self.users_list.append_column(column)
	
		column = gtk.TreeViewColumn('Users')
		column.pack_start(cell, True)
		column.add_attribute(cell, 'text', 1)
		column.set_resizable(True)
		column.set_sort_column_id(1)
		self.users_list.append_column(column)

		# Model
		liststore = gtk.ListStore(int, str)
		self.users_list.set_model(liststore)

		# Selection
		selection = self.users_list.get_selection()
		selection.set_mode(gtk.SELECTION_MULTIPLE)


	def populate_users_list(self):
		"""Populate the users list"""

		liststore = self.users_list.get_model()

		data = self.db.fetch_table('users')
		for row in data:
			liststore.append([row[0], row[1]])


	def setup_users_combo(self):
		"""Setup the users combobox"""

		cell = gtk.CellRendererText()
		self.users_combo.pack_start(cell, True)
		
		self.users_combo.add_attribute(cell, 'text', 0)
		
		liststore = gtk.ListStore(str, int)
		self.users_combo.set_model(liststore)


	def populate_users_combo(self):
		"""Populate the users combobox"""

		liststore = self.users_combo.get_model()

		data = self.db.fetch_table('users')
		for row in data:
			liststore.append([row[1], row[0]])


	def setup_activities_combo(self):
		"""Setup the activities combobox"""

		cell = gtk.CellRendererText()
		self.activities_combo.pack_start(cell, True)

		self.activities_combo.add_attribute(cell, 'text', 0)

		liststore = gtk.ListStore(str, int)
		self.activities_combo.set_model(liststore)


	def populate_activities_combo(self):
		"""Populate the activities combobox"""

		liststore = self.activities_combo.get_model()
		
		data = self.db.fetch_table('activities')
		for row in data:
			liststore.append([row[1], row[0]])


	def get_active_combo(self, combobox):
		"""Return the active text from the combobox"""

		model = combobox.get_model()
		active = combobox.get_active()
		if active < 0:
			return None
		return model[active][1]


	def get_presence(self):
		"""Return presence list"""
	
		presence_l = []

		selection = self.users_list.get_selection()
		(model, paths) = selection.get_selected_rows()

		for path in paths:
			presence_l.append(int(model[path][0]))

		return presence_l


	# Callbacks
	def on_selection_win_destroy(self, widget, data=None):

		self.table_win.destroy()
		return self.selection


	def on_ok_button_clicked(self, widget, data=None):
		
		self.selection['presence_l'] = self.get_presence()
		self.selection['presence'] = self.db.enc_presence(self.get_presence())
		self.selection['user'] = self.get_active_combo(self.users_combo)
		self.selection['activity'] = self.get_active_combo(self.activities_combo)
